import React from "react";
import { Conference } from 'between-api';
import { DEVICE_UPDATED, ERROR_STATUS, LOCAL_PARTICIPANT, LOCAL_STREAMS, PEER_JOINED, PEER_LEFT, REMOTE_STREAMS, SESSION_TERMINATED, VIDEO_CREATED, CONNECTED } from "./constant";
import WebRTCFooter from "./components/WebRTCFooter/WebRTCFooter";

const apiKey = 'API_HERE';
const conferenceId = '1001';

class VoipConference extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            local: null,
            remoteParticpants: {},
            isMuted: false,
            isVideoEnabled: true,
            audioInputs: [],
            videoInputs: [],
            locationType: '', // onsite OR offsite
            connected: false,
        }
        this.conference = new Conference(apiKey, conferenceId, null);
    }

    componentDidMount() {
        this.listeners();
    }

    listeners = () => {
        this.conference.on(LOCAL_PARTICIPANT, (local) => {
            this.setState({ local });

            local.on(VIDEO_CREATED, (video) => {
                console.log(LOCAL_PARTICIPANT, video)
                document.getElementById(LOCAL_STREAMS).append(video)
            })
        });

        this.conference.on(PEER_JOINED, (peer) => {
            const { remoteParticpants } = this.state;

            peer.on(VIDEO_CREATED, (video) => {
                console.log(video)
                document.getElementById(REMOTE_STREAMS).append(video)
            })

            this.setState({ ...remoteParticpants, [peer.id]: peer });
        });

        this.conference.on(PEER_LEFT, (peer) => {
            const { remoteParticpants } = this.state;
            let id = peer.id
            
            const copiedRemoteParticpants = { ...remoteParticpants };
            delete copiedRemoteParticpants[id];

            this.setState({ remoteParticpants: copiedRemoteParticpants });
            document.getElementById(id).remove();
        });

        this.conference.on(DEVICE_UPDATED, (devices) => {
          const { local } = this.state;

            const audioInputs = devices.filter((device) => device.kind === 'audioinput' && device.deviceId !== 'default')
            .map((audioInput) => ({ ...audioInput, label: audioInput.label, value: audioInput.deviceId }));

            const videoInputs = devices.filter((device) => device.kind === 'videoinput')
            .map((videoInput) => ({ ...videoInput, label: videoInput.label, value: videoInput.deviceId }));

            local?.setAudioSource?.(audioInputs[0].value);
            local?.publishVideo?.(videoInputs[0].value);

            this.setState({ audioInputs, videoInputs });
        })

        this.conference.on(SESSION_TERMINATED, () => {
            this.setState({ remoteParticpants: {}, local: null, connected: false, locationType: '' });
            document.getElementById("localStreams").innerHTML = '';
        })

        this.conference.on(CONNECTED, () => {
            this.setState({ connected: true })
        })
    }

    handleStartCall = async () => {
        try {
            const status = await this.conference.getPermissionState()
            if (ERROR_STATUS.includes(status.state)) {
                return alert(status.error.rawStr);
            }
            this.conference.setLocationType(this.state.locationType)
            await this.conference.startCall()
        } catch (e) {
            alert(`Error starting call. ${e.message}`)
        }
    }

    handleEndCall = async () => {
        try {
            this.conference.endCall()
        } catch (e) {
            console.log('end call error', e);
        }
    }

    handleToggleMute = () => {
        const { local, isMuted } = this.state;
        this.setState({ isMuted: !isMuted }, () => {
            local.publishAudio(!this.state.isMuted);
        })
    }

    handleToggleVideo = () => {
        const { local, isVideoEnabled } = this.state;
        this.setState({ isVideoEnabled: !isVideoEnabled }, () => {
            local.publishVideo(this.state.isVideoEnabled);
        });
    }

    handleToggleAudioSource = ({ value }) => {
      const { local } = this.state;
      console.log(value);
      local.setAudioSource(value);
    }
  
    handleToggleVideoSource = ({ value }) => {
      const { local } = this.state;
      console.log(value);
      local.setVideoSource(value)
    }

    // ui helpers

    // render selection buttons
    renderButtons = () => {
        const { connected, locationType } = this.state;

        if (connected) return null

        if (locationType.length > 0 && !connected) return (
            <div className="button-container">
                <p>Start your Conference</p>
                <div className="buttons">
                    <button onClick={this.handleStartCall}>Start Call</button>
                </div>
            </div>
        )

        return (
            <div className="button-container">
                <p>Select Location Type</p>
                <div className="buttons">
                    <button onClick={() => this.setState({ locationType: 'offsite' })}>Offsite</button>
                    <button onClick={() => this.setState({ locationType: 'onsite' })}>Onsite</button>
                </div>
            </div>
        )
    }

    render() {
        const { isMuted, isVideoEnabled, audioInputs, videoInputs, connected } = this.state;
        return (
            <div>
                <div className="main">
                    {this.renderButtons()}
                    <div id="localStreams" />
                    <div id="remoteStreams" />
                </div>
                <WebRTCFooter
                    isConnected={connected}
                    isMuted={isMuted}
                    audioInputs={audioInputs}
                    videoInputs={videoInputs}
                    isVideoEnabled={isVideoEnabled}
                    handleToggleAudioSource={this.handleToggleAudioSource}
                    handleToggleVideoSource={this.handleToggleVideoSource}
                    handleToggleMute={this.handleToggleMute}
                    handleToggleVideo={this.handleToggleVideo}
                    handleStartCall={this.handleStartCall}
                    handleEndCall={this.handleEndCall}
                />
            </div>
        )
    }
}

export default VoipConference;